// file: $pdsp/src/v1.0/class/fourier_transform/fourier_transform_constants.h
//
// this file defines the fourier_transform class's default data 
//

// make sure definitions are only made once
//
#ifndef __FOURIER_TRANSFORM_CONSTANTS
#define __FOURIER_TRANSFORM_CONSTANTS

// isip include files
//
#include <math/integral/integral_constants.h>

// define the class name
//
#define FT_CLASS_NAME "Fourier_transform"

// direction flags
//
#define FT_FORWARD (int_4)0
#define FT_REVERSE (int_4)1

// type of data flags
//
#define FT_REAL (int_4)0
#define FT_COMPLEX (int_4)1

// define the available algorithms
//
#define FT_DFT_NAME (char_1*)"dft"
#define FT_QFT_NAME (char_1*)"qft"
#define FT_RADIX_2_NAME (char_1*)"radix_2"
#define FT_RADIX_4_NAME (char_1*)"radix_4"
#define FT_FAST_HARTLEY_NAME (char_1*)"fht"
#define FT_SRFFT_NAME (char_1*)"srfft"

#define FT_DFT_REAL (int_4)0
#define FT_DFT_COMPLEX (int_4)1
#define FT_RADIX_2_REAL (int_4)2
#define FT_RADIX_4_REAL (int_4)3
#define FT_RADIX_2_COMPLEX (int_4)4
#define FT_RADIX_4_COMPLEX (int_4)5
#define FT_FAST_HARTLEY_REAL (int_4)6
#define FT_FAST_HARTLEY_COMPLEX (int_4)7
#define FT_QFT_REAL (int_4)8
#define FT_QFT_COMPLEX (int_4)9
#define FT_SRFFT_REAL (int_4)10
#define FT_SRFFT_COMPLEX (int_4)11

// define default values for the parameters
//
#define FT_DEFAULT_ORDER 1024
#define FT_DEFAULT_DATA_TYPE "complex"
#define FT_DEFAULT_NUM_ITER 1
#define FT_DEFAULT_DBG_LEVEL 0
#define FT_DEFAULT_STATS 0
#define FT_DEFAULT_ALGO_NAME "radix_2"
 
// define command line identifiers
//
#define FT_OPT_ORDER "-order"
#define FT_OPT_DATA_TYPE "-type"
#define FT_OPT_NUM_ITER "-niter"
#define FT_OPT_DBG_LEVEL "-debug"
#define FT_OPT_STATS "-stats"
#define FT_OPT_ALGO_NAME "-algo"
#define FT_OPT_HELP "-help"
 
// define help file name
//
#define FT_HELP_FILE_NAME "/isip/d00/parallel_dsp/util/v1.0/compare_algorithms/fourier_transform.help"
 

// define precomputed arithmetic values
//
#define SQRT2_2   0.70710678118654752440084436210484
#define SQRT2   2*0.70710678118654752440084436210484

// end of file
// 
#endif
