// file: $pdsp/src/v1.0/class/fourier_transform/ft_comp_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: compute_cc
//
// arguments:
// float_4* out_a : output array
// float_4* in_a : input array
//
// return: a logical_1 value indicating status
//
logical_1 Fourier_transform::compute_cc(float_4* out_a, float_4* in_a) {

  // branch on the algorithm
  //
  if (algorithm_d == FT_DFT_REAL) {
    return dft_real_cc(out_a, in_a);
  }

  if (algorithm_d == FT_DFT_COMPLEX) {
    return dft_complex_cc(out_a, in_a);
  }
  
  else if (algorithm_d == FT_RADIX_2_COMPLEX) {
    return fft2_complex_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_RADIX_2_REAL) {
    return fft2_real_cc(out_a, in_a);
  }
  
  else if (algorithm_d == FT_RADIX_4_COMPLEX) {
    return fft4_complex_cc(out_a, in_a);
  }

  /*
  else if (algorithm_d == FT_QFT_REAL) {
    return qft_real_cc(out_a, in_a);
  }
  
  else if (algorithm_d == FT_QFT_COMPLEX) {
    return qft_complex_cc(out_a, in_a);
  }
  */
  
  else if (algorithm_d == FT_FAST_HARTLEY_COMPLEX) {
    return fht_complex_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_FAST_HARTLEY_REAL) {
    return fht_real_cc(out_a);
  }

  else if (algorithm_d == FT_SRFFT_COMPLEX) {
    return srfft_complex_cc(out_a,in_a);
   }

  else if (algorithm_d == FT_SRFFT_REAL) {
    return srfft_real_cc(out_a,in_a);
   }
  
  else {
    return ISIP_FALSE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
