// file: $pdsp/src/v1.0/class/fourier_transform/ft_dft_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: dft_real_cc
//
// arguments:
//  float_4* out_a : output data array
//  float_4* in_a : input data array
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::dft_real_cc(float_4* out_a, float_4* in_a) {
  
  // start clock for timing
  //
  cpu_time_d = clock();
  
  // loop over all frequency samples
  //
  for (int_4 k=0; k<N_d; k++) {
    
    // declare local accumulators
    //
    float_4 sumr = (float_4)0.0;
    float_4 sumi = (float_4)0.0;
    
    // loop over the data
    //
    for(int_4 n=0; n<N_d; n++) {
      
      // compute the argument
      //
      float_4 arg = ISIP_TWOPI * (float_4)n * (float_4)k / (float_4)N_d;
      
      // compute the real part
      //
      sumr = mult_sum_cc(in_a[k], cos(arg), sumr);

      // compute the imaginary part
      //
      sumi = mult_sum_cc(-1.0*in_a[k], sin(arg), sumi);
    }
    
    // compute the squared magnitude
    //
    out_a[k] = mult_cc(sumr,sumr);
    out_a[k] = mult_sum_cc(sumi,sumi,out_a[k]);
  }

  // stop clock to end timing process
  //
  cpu_time_d = clock();
  
  // exit gracefully
  //
  return ISIP_TRUE;
}



