// file: $pdsp/src/v1.0/class/fourier_transform/ft_fft2_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// method: fft2_real_cc
//
// arguments:
//  float_4* output_a : output data array
//  float_4* input_a : input data array
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::fft2_real_cc( float_4* output_a,
					   float_4* input_a){
  
  // declare local variables
  //
  int_4 m,l,i1,i2;
  int_4 n1,n2;
  float_4 real_twid,imag_twid;
  float_4 tempr, tempi; 
  
  // get the lookup values
  //
  float_4 *wr = new float_4[N_d];
  float_4 *wi = new float_4[N_d];
  
  // create lookup table
  //
  lookup_cc(wr , wi);

  // start clock for timing
  //
  cpu_time_d = clock();
  
  m = (int)(log10(N_d)/log10(2));
  n2 = N_d;
  for(int i=1; i<m+1; i++)
    {
      n1 = n2;
      n2 = (int_4)(n2/2);
      i1 = 1;
      i2 = (int_4)((N_d)/n1);
      for(int j=1; j<n2+1; j++)
	{
	  real_twid = wr[i1];
	  imag_twid = wi[i1];
	  
	  for(int k = j; k <= N_d; k = k + n1)
	    {
	      l = k + n2;
	      tempi = add_cc(input_a[k],-1*input_a[l]);
	      tempr = add_cc(-1*output_a[l],output_a[k]);
	      output_a[k] = add_cc(output_a[k],output_a[l]);
	      input_a[k] = add_cc(input_a[k],input_a[l]);
	      output_a[l] = mult_cc(real_twid,tempr);
	      input_a[l] =  mult_cc(-1*imag_twid,tempr);
	    }
	  i1= (int_4)add_cc(i1,i2);
	}
    }

  // find the magnitude squared of the spectrum
  //
  for (int k= 0; k<=N_d; k++)
    {
      output_a[k] = mult_cc(output_a[k],output_a[k]);
      output_a[k] = mult_sum_cc(input_a[k],input_a[k],output_a[k]);
    }
  
  //procedure for bit reversal
  //
  float_4 temp = 0;
  int j = 1;
  for (int i = 1; i < N_d; i++)
    {
      if (i < j)
	{
	  temp = output_a[j];
	  output_a[j] = output_a[i];
	  output_a[i] = temp;
	  temp = input_a[j];
	  input_a[j] = input_a[i];
	  input_a[i] = temp;
	}
      int_4 k = N_d/2;
      while (k < j)
	{
	  j = j - k;
	  k = k/2;
	}
      j = j + k;
    }

  //stop clock for timing information
  //
  cpu_time_d = clock();
  
  // exit gracefully
  //
  return ISIP_TRUE;
}

