// file: $pdsp/src/v1.0/class/fourier_transform/ft_fft2_1.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
#include <math/integral/integral_constants.h>

// method: fft2_complex_cc
//
// This method is a fortran code converted to fit our requirements.
// The code follows the exact structure of that in the book by
// Proakis and Manolakis " Digital Signal Processing " pp.
//
// arguments:
//  float_4* in_r_a : output real array
//  float_4* in_i_a : input imaginary array
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::fft2_complex_cc( float_4* in_r_a,
					      float_4* in_i_a){
  

  // declare local variables
  //
  int_4 m,l,i1,i2;
  int_4 n1,n2;
  float_4 real_twid,imag_twid;
  float_4 tempr, tempi; 
  
  
  // get the lookup values
  //
  float_4 *wr = new float_4[N_d+1];
  float_4 *wi = new float_4[N_d+1];
  
  // create lookup table
  //
  lookup_cc(wr , wi);
  
  // start clock for timing
  //
  cpu_time_d = clock();
  
  m = powof2_cc();
  n2 = N_d;
  
  for(int i = 0; i < m; i++){
    n1 = n2;
    n2 = (int_4)(n2/2.0);
    i1 = 1;
    i2 = (int_4)((N_d)/(float_4)n1);
    
    for(int j = 0; j < n2; j++){
      real_twid = wr[i1];
      imag_twid = wi[i1];
      i1= (int_4)add_cc(i1,i2);
      
      for(int k = j; k < N_d; k = k + n1){
	l = k + n2;
	tempr = sub_cc(in_r_a[k],in_r_a[l]);
	in_r_a[k] = add_cc(in_r_a[k],in_r_a[l]);
	tempi = sub_cc(in_i_a[k],in_i_a[l]);
	in_i_a[k] = add_cc(in_i_a[k],in_i_a[l]);
	in_r_a[l] = add_cc(mult_cc(real_twid,tempr),
			   mult_cc(imag_twid,tempi));
	in_i_a[l] = sub_cc(mult_cc(real_twid,tempi),
			   mult_cc(imag_twid,tempr));
      }
    }
  }
  
  
  //stop clock for timing information
  //
  cpu_time_d = clock();

  //procedure for bit reversal
  //
  float_4 temp = 0;
  int j = 1;
  n1 = N_d - 1;
  for (int i = 1; i < n1; i++){
    if (i < j)
      {
	temp = in_r_a[j];
	in_r_a[j] = in_r_a[i];
	in_r_a[i] = temp;
	temp = in_i_a[j];
	in_i_a[j] = in_i_a[i];
	in_i_a[i] = temp;
      }
    int_4 k = N_d>>1;
    while (k < j){
      j = j - k;
      k = k>>1;
    }
    j = j + k;
  }

  // free memory
  //
  delete wr;
  delete wi;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}

