// file: $pdsp/src/v1.0/class/fourier_transform/ft_fft2_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
#include <math/integral/integral_constants.h>

// method: fft4_complex_cc
//
// This method is a fortran code converted to fit our requirements.
// The code follows the exact structure of that in the book by
// Burrus and Parks "DFT/FFT and Convolution Algorithms". 
//
// arguments:
//  float_4* real_a : real part of data 
//  float_4* imag_a : imaginary part of data 
//
// return: a logical_1 value indicating status
//
// implements a complex radix-4 fast  fourier transform
//
logical_1 Fourier_transform::fft4_complex_cc( float_4* real_a,
					      float_4* imag_a){
  
  // declare local variables
  //
  int_4 m,i1,i2,i3;
  int_4 n1,n2;
  float_4 a,b,c,e,r1,r2,r3,r4,s1,s2,s3,s4,co1,co2,co3,si1,si2,si3;
  
  // start clock for timing
  //
  cpu_time_d = clock();
  
  m = powof2_cc();
  n2 = N_d;
  
  for(int k = 0; k < m; k++){
    n1 = n2;
    n2 = n2>>2;
    e = (float_4)(ISIP_TWOPI/(float_4)n1);
    a = 0.0;
    
    for(int j = 0; j < n2; j++){
      b = add_cc(a,a);
      c = add_cc(a,b);
      co1 = cos(a);
      co2 = cos(b);
      co3 = cos(c);
      si1 = sin(a);
      si2 = sin(b);
      si3 = sin(c);
      a = mult_cc((float_4)j,e);
      
      for(int i = j; i < N_d; i+=n1){
	i1 = (int_4)add_cc(i,n2);
	i2 = (int_4)add_cc(i1,n2);
	i3 = (int_4)add_cc(i2,n2);
	r1 = add_cc(real_a[i],real_a[i2]);
	r3 = sub_cc(real_a[i],real_a[i2]);
	s1 = add_cc(imag_a[i],imag_a[i2]);
	s3 = sub_cc(imag_a[i],imag_a[i2]);
	r2 = add_cc(real_a[i1],real_a[i3]);
	r4 = sub_cc(real_a[i1],real_a[i3]);
	s2 = add_cc(imag_a[i1],imag_a[i3]);
	s4 = sub_cc(imag_a[i1],imag_a[i3]);
	real_a[i] = add_cc(r1,r2);
	r2 = sub_cc(r1,r2);      
	r1 = sub_cc(r3,s4);
	r3 = add_cc(r3,s4);      
	imag_a[i] = add_cc(s1,s2);
	s2 = sub_cc(s1,s2);      
	s1 = add_cc(s3,s4);
	s3 = sub_cc(s3,s4);      
	real_a[i1] = add_cc(mult_cc(r3,co1),mult_cc(s3,si1));
	imag_a[i1] = sub_cc(mult_cc(s3,co1),mult_cc(r3,si1));
	real_a[i2] = add_cc(mult_cc(r2,co2),mult_cc(s2,si2));
	imag_a[i2] = sub_cc(mult_cc(s2,co2),mult_cc(r2,si2));
	real_a[i3] = add_cc(mult_cc(r1,co3),mult_cc(s1,si3));
	imag_a[i3] = sub_cc(mult_cc(s1,co3),mult_cc(r1,si3));
      }
    }
  }
  
  // stop clock for timing information
  //
  cpu_time_d = clock();


  //procedure for bit reversal
  //
  float_4 temp = 0;
  int j = 1;
  for (int i = 1; i < N_d - 1; i++){
    if (i < j){
      temp = real_a[j];
      real_a[j] = real_a[i];
      real_a[i] = temp;
      temp = imag_a[j];
      imag_a[j] = imag_a[i];
      imag_a[i] = temp;
    }
    int_4 k = N_d>>2;
    while (k*3 < j){
      j = j - k*3;
      k = k>>2;
    }
    j = j + k;
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}

