// file: $pdsp/src/v1.0/class/fourier_transform/ft_fht_1.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: fht_complex_cc
//
// This code is based on the algorithm as described in the work by
// Ron Mayer whose code is available as a free-ware. 
//
// arguments:
//  float_4* real_a : real data array
//  float_4* imag_a : imag data array
//
// return: a logical_1 value indicating status
//
// implements a complex Fast Hartley transform
//
logical_1 Fourier_transform::fht_complex_cc(float_4* real_a, float_4* imag_a) {
  
  float_4 a,b,c,d;
  int_4 i,j,k;
  float_4 q,r,s,t;


  // start clock for timing the algorithm
  //
  cpu_time_d = clock();

  // prepare data for FHT computations
  
  for (i = 1,j = (N_d - 1), k = (N_d/2); i < k; i++,j--) {
    a = real_a[i];
    b = real_a[j];
    q = add_cc(a,b);
    r = sub_cc(a,b);
    c = imag_a[i];
    d = imag_a[j];
    s = add_cc(c,d);
    t = sub_cc(c,d);
    real_a[i] = mult_cc(add_cc(q,t),0.5);
    real_a[j] = mult_cc(sub_cc(q,t),0.5);
    imag_a[i] = mult_cc(sub_cc(s,r),0.5);
    imag_a[j] = mult_cc(add_cc(s,r),0.5);
  }

  // compute hartley transform of the real and imaginary parts separately
  //
  fht_real_cc(real_a);
  fht_real_cc(imag_a);
  
  // stop clock to get the timing information
  //
  cpu_time_d = clock();
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
