// file: $pdsp/src/v1.0/class/fourier_transform/ft_lookup_1.cc
//
 
// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
 
// method: srfft_new_lookup_cc
//
// This code is based on the code by Sitton of the Rice University.
//
// arguments:
//  float_4* cos_a : cosine of the required angle
//  float_4* sin_a : sine of the required angle
//  
// return: logical_1 indicating no error
//
// routine used to create lookup table need for SRFFT computation
//
logical_1 Fourier_transform::srfft_new_lookup_cc(float_4* cos_a,
						 float_4* sin_a){

	float_4  t, t0, t1, t2, t3;
	int_4     i, n2, n4;

	t = ISIP_TWOPI/N_d;
	n2 = N_d >> 1;
	n4 = n2 >> 1;
	sin_a[N_d-n4] = cos_a[n2] = -(sin_a[n4] = cos_a[0] = t0 = 1.0);
	cos_a[N_d-n4] = cos_a[n4] = sin_a[n2] = sin_a[0] = t1 = 0.0;

	t2 = cos(t);
	t3 = sin(t);

/*  Fill the cos-sin tables     */

	for (i = 1; i < n4; i++)
	{
		cos_a[n2+i] = cos_a[n2-i] = -(cos_a[N_d-i] = cos_a[i] = t = t0*t2 - t1*t3);
		sin_a[n2+i] = sin_a[N_d-i] = -(sin_a[n2-i] = sin_a[i] = t1 = t0*t3 + t1*t2);
		t0 = t;
	}
	
	// exit gracefully
	//
	return ISIP_TRUE;     
}

