// file: $PDSP/class/fourier_transform/v1.1/ft_dft_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: dft_complex_cc
//
// arguments:
// float_8* output_a :(output) output data array 
//			length of output array will always be 2*N
//			memory is allocated internally not by the calling
//			program.
// float_8* input_a : (input) input data array
//			length of input array is N for real data
//			length of input array is 2*N for complex data.
//			input data memory should be allocated by the
//			calling program.
//
// return: a logical_1 value indicating status
//
// implements a complex discrete fourier transform
//
logical_1 Fourier_transform::dft_complex_cc(float_8* output_a,
					    float_8* input_a) {

  // loop over all frequency samples
  //
  for (int_4 k = 0; k < N_d; k = add_cc(k,1)) {
    
    // declare local accumulators
    //
    float_8 sumr((float_8)0.0);
    float_8 sumi((float_8)0.0);
    
    // loop over the data
    //
    for(int_4 n = 0; n < N_d; n = add_cc(n,1)) {
      
      // compute the argument
      //
      float_8 arg(ISIP_TWOPI * (float_8)n * (float_8)k / (float_8)N_d);
      
      // compute the real part
      //
      float_8 tempr = (mult_cc(input_a[2*n+1], sin(arg)));
      tempr = mult_sum_cc(input_a[2*n],cos(arg),tempr);
      sumr = add_cc(tempr,sumr);
      
      // compute the imaginary part
      //
      float_8 tempi = (mult_cc(input_a[2*n+1], cos(arg)));
      tempi = mult_sum_cc(input_a[2*n],-sin(arg),tempi);
      sumi = add_cc(tempi,sumi);
    }

    
    // store the output data in interlaced format
    //
    output_a[2*k] = sumr;
    output_a[2*k+1] = sumi;
  }

  if ( ISIP_DEBUG_NONE < debug_level_d) {
    
    // diagnostic printing
    //
    for (int_4 kk = 0; kk < N_d; kk++) {
      printf("output[%ld] = %f    + j %f\n", kk, output_a[2*kk],
	     output_a[2*kk+1]);
    }
    
  }
    
  // exit gracefully
  //
  return ISIP_TRUE;
}
