// file: $PDSP/class/fourier_transform/v1.1/ft_ditf_1.cc
//

// include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: ditf_complex_cc
//
// This method is based on the DITF algorithm by Ali Saidi
// found in the 1994 ICASSP Proceedings 
//
// arguments:
// float_8* output_a :(output) output data array 
//                      length of output array will always be 2*N
//                      memory is allocated internally, not by the calling
//                      program.
// float_8* input_a : (input) input data array
//                      length of input array is 2*N
//                      memory is allocated by the calling
//                      program.
//
// return: logical_1 indicating status
//
// this method performs the complex ditf algorithm
//
logical_1 Fourier_transform::ditf_complex_cc(float_8* output_a,
					     float_8* input_a) {
  // declare local variables
  //
  int_4 i;			  // incremental counter index
  float_8 output_temp_a[2*N_d];
  float_8 input_temp_a[2*N_d];
  float_8 temp[N_d];

  // copy input imaginary data to temporary memory and input real data
  // into output so that the input data is retained
  // after calculations.
  //
  for (int_4 kk = 0; kk < N_d; kk++) {
    temp[kk] = input_a[2*kk+1];
    output_a[kk] = input_a[2*kk];
  }

  // calculate 0..N/2-1 ditf coefficients
  //
  ditf_real_cc(output_temp_a, output_a);
  ditf_real_cc(input_temp_a, temp);

  // interlace the output into output_a
  //
  for (i = 0; i < N_d; i = add_cc(i,1)){
    output_a[2*i] = sub_cc(output_temp_a[2*i],input_temp_a[2*i+1]);
    output_a[2*i+1]= add_cc(output_temp_a[2*i+1],input_temp_a[2*i]);
  }
  
  if ( ISIP_DEBUG_NONE < debug_level_d) {
    
    // diagnostic printing
    //
    for (int_4 kk = 0; kk < N_d; kk++) {
      printf("output[%ld] = %f    + j %f\n", kk, output_a[2*kk],
	     output_a[2*kk+1]);
    }
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}





