// file: $PDSP/class/fourier_transform/v1.1/fourier_transform_constants.h
//
// this file defines the fourier_transform class's default data 
//

// make sure definitions are only made once
//
#ifndef __FOURIER_TRANSFORM_CONSTANTS
#define __FOURIER_TRANSFORM_CONSTANTS

// isip include files
//
#include <integral_constants.h>

// define the class name
//
#define FT_CLASS_NAME "Fourier_transform"

// direction flags
//
#define FT_FORWARD (int_4)0
#define FT_REVERSE (int_4)1

// type of data flags
//
#define FT_REAL (int_4)0
#define FT_COMPLEX (int_4)1

// define the values corresponding to each algorithm
//
#define FT_DFT (int_4)0
#define FT_DITF (int_4)1
#define FT_FHT (int_4)2
#define FT_QFT (int_4)3
#define FT_RADIX_2 (int_4)4
#define FT_RADIX_4 (int_4)5
#define FT_SRFFT (int_4)6

// define the strings corresponding to each algorithm
//
#define FT_DFT_NAME (char_1*)"dft"
#define FT_DITF_NAME (char_1*)"ditf"
#define FT_FHT_NAME (char_1*)"fht"
#define FT_QFT_NAME (char_1*)"qft"
#define FT_RADIX_2_NAME (char_1*)"radix_2"
#define FT_RADIX_4_NAME (char_1*)"radix_4"
#define FT_SRFFT_NAME (char_1*)"srfft"


// define values corresponding to each algorithm
// and the data type
//
#define FT_DFT_REAL (int_4)0
#define FT_DFT_COMPLEX (int_4)1
#define FT_DITF_REAL (int_4)2
#define FT_DITF_COMPLEX (int_4)3
#define FT_FHT_REAL (int_4)4
#define FT_FHT_COMPLEX (int_4)5
#define FT_QFT_REAL (int_4)6
#define FT_QFT_COMPLEX (int_4)7
#define FT_RADIX_2_REAL (int_4)8
#define FT_RADIX_2_COMPLEX (int_4)9
#define FT_RADIX_4_REAL (int_4)10
#define FT_RADIX_4_COMPLEX (int_4)11
#define FT_SRFFT_REAL (int_4)12
#define FT_SRFFT_COMPLEX (int_4)13


// define default algorithm names for real and complex input
//
#define FT_DEFAULT_REAL_ALGO FT_RADIX_2_REAL
#define FT_DEFAULT_COMPLEX_ALGO FT_RADIX_2_COMPLEX

// define default values for the parameters
//
#define FT_DEFAULT_ORDER (int_4)1024
#define FT_DEFAULT_DATA_TYPE (int_4)FT_COMPLEX
#define FT_DEFAULT_DBG_LEVEL (int_4)0
#define FT_DEFAULT_ALGO_NAME (int_4)FT_SRFFT
#define FT_DEFAULT_INPUT_SIZE (int_4)8192

// define the levels of statistics
//
#define FT_STAT_LEVEL_NONE ISIP_DEBUG_NONE
#define FT_STAT_LEVEL_BRIEF ISIP_DEBUG_BRIEF
#define FT_STAT_LEVEL_DETAILED ISIP_DEBUG_DETAILED
#define FT_STAT_LEVEL_FULL ISIP_DEBUG_FULL

// define constant to indicate error in return of functions
//
#define FT_ERROR (int_4)-1

// define format string constants
//
#define FT_CPU_FORMAT "the average cpu time taken per iteration:\t%10.1f us\n\n"
#define FT_OPER_FORMAT "the number of %s per iteration:\t%ld\n"
#define FT_MEM_FORMAT "the amount of memory used per iteration:\t%ld\n"

// shifting constants
//
#define FT_RIGHT (int_4)0
#define FT_LEFT (int_4)1

// end of file
// 
#endif



