// file: $PDSP/class/fourier_transform/v1.1/ft_check_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: algorithm_check_cc
//
// arguments:
//  none.
//
// return: a logical_1 value indicating status
//
logical_1 Fourier_transform::algorithm_check_cc() {

  // determine if N_d is a power of 2
  //
  int_4 m = powof2_cc();

  // check if N_d is a power of two
  //
  if (m == FT_ERROR) {
    return ISIP_FALSE;
  }

  if ((m%2) != 0) {
    if (algorithm_d == FT_RADIX_4_REAL) {
      algorithm_d = FT_DEFAULT_REAL_ALGO;
    }
    else if (algorithm_d == FT_RADIX_4_COMPLEX) {
      algorithm_d = FT_DEFAULT_COMPLEX_ALGO;
    }
  }

  // assure that the order is greater than 16 for certain algorithms
  // these algorithms are optimized for orders 16 and above
  // these include:  fht, and qft.
  //
  if (N_d < 16) {
    if (algorithm_d == FT_FHT_REAL) {
      algorithm_d = FT_DEFAULT_REAL_ALGO;
    }

    else if (algorithm_d == FT_FHT_COMPLEX) {
      algorithm_d = FT_DEFAULT_COMPLEX_ALGO;
    }
    
    else if (algorithm_d == FT_QFT_REAL) {
      algorithm_d = FT_DEFAULT_REAL_ALGO;
    }
    
    else if (algorithm_d == FT_QFT_COMPLEX) {
      algorithm_d = FT_DEFAULT_COMPLEX_ALGO;
    }
  }
    
  // exit gracefully
  //
  return ISIP_TRUE;
}

