// file: $PDSP/class/fourier_transform/v1.1/ft_comp_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: compute_cc
//
// arguments:
// float_4* output_a :(output) output data array 
//			length of output array will always be 2*N
//			memory is allocated internally not by the calling
//			program.
// float_4* input_a : (input) input data array
//			length of input array is N for real data
//			length of input array is 2*N for complex data.
//			input data memory should be allocated by the
//			calling program.
//
// return: a logical_1 value indicating status
//
// this method calls the fft algorithm chosen by the user
// calls either complex or real algorithm, which ever is desired
//
logical_1 Fourier_transform::compute_cc(float_4* output_a, float_4* input_a) {

  // check the order of the algorithm and assign a default algorithm if
  // necessary
  //
  algorithm_check_cc();
  
  // branch on the algorithm
  //
  if (algorithm_d == FT_DFT_REAL) {
    return dft_real_cc(output_a, input_a);
  }

  if (algorithm_d == FT_DFT_COMPLEX) {
    return dft_complex_cc(output_a, input_a);
  }
  
  else if (algorithm_d == FT_RADIX_2_COMPLEX) {
    return fft2_complex_cc(output_a, input_a);
  }

  else if (algorithm_d == FT_RADIX_2_REAL) {
    return fft2_real_cc(output_a, input_a);
  }

  else if (algorithm_d == FT_RADIX_4_REAL) {
    return fft4_real_cc(output_a, input_a);
  }
  
  else if (algorithm_d == FT_RADIX_4_COMPLEX) {
    return fft4_complex_cc(output_a, input_a);
  }

  else if (algorithm_d == FT_FHT_COMPLEX) {
    return fht_complex_cc(output_a, input_a);
  }

  else if (algorithm_d == FT_FHT_REAL) {
    return fht_real_cc(output_a, input_a);
  }

  else if (algorithm_d == FT_SRFFT_COMPLEX) {
    return srfft_complex_cc(output_a,input_a);
   }

  else if (algorithm_d == FT_SRFFT_REAL) {
    return srfft_real_cc(output_a,input_a);
   }

  else if (algorithm_d == FT_QFT_REAL) {
    return qft_real_cc(output_a,input_a);
   }

  else if (algorithm_d == FT_QFT_COMPLEX) {
    return qft_complex_cc(output_a,input_a);
  }
  
  else if (algorithm_d == FT_DITF_COMPLEX) {
    //remove me
    //
    logical_1 temp;
    ditf_init_cc();
    temp = ditf_complex_cc(output_a, input_a);
    ditf_cleanup_cc();
    return temp;
  }

  else if (algorithm_d == FT_DITF_REAL) {
    //remove me
    //
    logical_1 temp;
    ditf_init_cc();
    temp = ditf_real_cc(output_a, input_a);
    ditf_cleanup_cc();
    return temp;
  }
  
  else {
    return ISIP_FALSE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
