// file: $PDSP/class/fourier_transform/v1.1/ft_dft_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include <float_4.h>

#include "fourier_transform_constants.h"

// method: dft_real_cc
//
// arguments:
// float_4* output_a :(output) output data array 
//			length of output array will always be 2*N
//			memory is allocated internally not by the calling
//			program.
// float_4* input_a : (input) input data array
//			length of input array is N
//			input data memory should be allocated by the
//			calling program.
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::dft_real_cc(float_4* output_a, float_4* input_a) {
  
  // loop over all frequency samples
  //
  for (int_4 k=0; k<N_d; k = add_cc(k,1)) {
    
    // declare local accumulators
    //
    Float_4 sumr((float_4)0.0);
    Float_4 sumi((float_4)0.0);
    
    // loop over the data
    //
    for(int_4 n = 0; n < N_d; n = add_cc(n,1)) {
      
      // compute the argument
      //
      Float_4 arg(ISIP_TWOPI * (float_4)n * (float_4)k / (float_4)N_d);
      
      // compute the real part
      //
      sumr = mult_sum_cc(input_a[n], cos((float_4)arg), (float_4)sumr);
      
      // compute the imaginary part
      //
      sumi = mult_sum_cc(-input_a[n], sin((float_4)arg), (float_4)sumi);
    }
    
    // put the output data in an interlaced format
    //
    output_a[2*k] = (float_4)sumr;
    output_a[2*k+1] = (float_4)sumi;

    // set the memory used
    //
    memory_usage_d = sumr.get_ref_count_cc();
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
