// file: $PDSP/class/fourier_transform/v1.1/ft_srfft_0.cc
//
 
// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
#include <float_4.h>

// global constants
//
#define DSQC    0.707106781186547f

// method: srfft_real_cc
//
// This code is based on the code by Sitton of the Rice University.
// remove me
//
// arguments:
// float_4* output_a :(output) output data array 
//                      length of output array will always be 2*N
//                      memory is allocated internally, not by the calling
//                      program.
// float_4* input_a : (input) input data array
//                      length of input array is N
//                      memory is allocated by the calling
//                      program.
//
// return: a logical_1 value indicating status
//
// routine used to compute the real  split-radix fft
//
logical_1 Fourier_transform::srfft_real_cc(float_4* output_a,
					   float_4* input_a) {
  Float_4   t1, t2, t3, t4, t5, t6, t;
  Float_4   cc1, ss1, cc3, ss3;
  int_4     i, j, k, l, n1, n2, n4, n8, inc;
  int_4     is, id, i1, i2, i3, i4, i5, i6, i7, i8;
  Float_4   real_a[N_d];

  // create sin-cos lookup table
  //
  Float_4* c = new Float_4[N_d];
  Float_4* s = new Float_4[N_d];

  srfft_new_lookup_cc((float_4*)c,(float_4*)s);

  int_4 m = powof2_cc();
  
  // copy input data to temporary memory so that the input data is retained
  // after calculations.
  //
  for (int_4 kk = 0; kk < N_d; kk++) {
    real_a[kk] = input_a[kk];
    output_a[kk] = 0;
  }

  // bit reversal routine
  //
  n1 = sub_cc(N_d,1);
  n2 = shift_cc(FT_RIGHT,N_d, 1);
  for (i = j = 0; i < n1; i = add_cc(i,1), j = add_cc(j,k)){
    if (i < j){
      t = real_a[j];
      real_a[j] = real_a[i];
      real_a[i] = t;
    }

    k = n2;
    while (k <= j){
      j -= k;
      k = shift_cc(FT_RIGHT,k,1);
    } 
  }
  
  //  First (twiddleless) L Butterflies   
  //
  is = 0;
  id = 4;
  n1 = sub_cc(N_d,1);
  
  do {
      for (i = is; i < n1; i = add_cc(i,id)){
	  t1 = real_a[i];
	  i1 = add_cc(i,1);
	  t2 = real_a[i1];
	  real_a[i] = add_cc(t1,t2);
	  real_a[i1] = sub_cc(t1,t2);
	}
      is = sub_cc((shift_cc(FT_LEFT,id,1)),2);
      id = shift_cc(FT_LEFT,id,2);
    } while (is < n1) ;
  
  //  Other (twiddled) L Butterflies  
  //
  n2 = 2;
  for (k = 2; k <= m; k = add_cc(k,1)){
      n4 = shift_cc(FT_RIGHT,n2,1);
      n8 = shift_cc(FT_RIGHT,n4,1);
      n2 = shift_cc(FT_LEFT,n2,1);
      is = 0;
      id = shift_cc(FT_LEFT,n2,1);

      do {
	  for (i = is; i < N_d; i = add_cc(i,id)){
	      i2 = (add_cc(i,n4));
	      i3 = (add_cc(i2,n4));
	      i4 = (add_cc(i3,n4));
	      t3 = real_a[i3];
	      t4 = real_a[i4];
	      t1 = real_a[i];
	      t2 = add_cc(t4,t3);
	      real_a[i4] = sub_cc(t4,t3);
	      real_a[i3] = sub_cc(t1,t2);
	      real_a[i] = add_cc(t1,t2);
	      
	      if (n4 != 1) {
		  i1 = add_cc(i,n8);
		  i2 = add_cc(n8,i2);
		  i3 = add_cc(n8,i3);
		  i4 = add_cc(n8,i4);
		  t3 = real_a[i3];
		  t4 = real_a[i4];
		  t1 = mult_cc(add_cc(t3,t4),DSQC);
		  t2 = mult_cc(sub_cc(t3,t4),DSQC);
		  t3 = real_a[i1];
		  t4 = real_a[i2];
		  real_a[i4] = sub_cc(t4,t1);
		  real_a[i3] = -add_cc(t4,t1);
		  real_a[i2] = sub_cc(t3,t2);
		  real_a[i1] = add_cc(t3,t2);
		}
	    }
	  is = sub_cc(shift_cc(FT_LEFT,id,1),n2);
	  id = shift_cc(FT_LEFT,id,2);
	} while (is < n1);
      
      inc = N_d/n2;
      l = inc;
      for (j = 1; j < n8; j = add_cc(j,1)){
	  cc1 = c[l];
	  ss1 = s[l];
	  i1 = mult_cc(l,3);
	  cc3 = c[i1];
	  ss3 = s[i1];
	  l = mult_cc(add_cc(j,1),inc);
	  is = 0;
	  id = shift_cc(FT_LEFT,n2,1);
	  do
	    {
	      for (i = is; i < N_d; i = add_cc(i,id)){
		  i1 = add_cc(i, j);
		  i2 = add_cc(i1, n4);
		  i3 = add_cc(i2, n4);
		  i4 = add_cc(i3, n4);
		  i5 = add_cc(sub_cc(i,j),n4);
		  i6 = add_cc(i5, n4);
		  i7 = add_cc(i6, n4);
		  i8 = add_cc(i7, n4);
		  t3 = real_a[i3];
		  t4 = real_a[i7];
		  t1 = add_cc(mult_cc(t3,cc1),mult_cc(t4,ss1));
		  t2 = sub_cc(mult_cc(t4,cc1),mult_cc(t3,ss1));
		  t5 = real_a[i4];
		  t6 = real_a[i8];
		  t3 = add_cc(mult_cc(t5,cc3),mult_cc(t6,ss3));
		  t4 = sub_cc(mult_cc(t6,cc3),mult_cc(t5,ss3));
		  t5 = add_cc(t1, t3);
		  t3 = sub_cc(t1,t3);
		  t6 = add_cc(t2,t4);
		  t4 = sub_cc(t2,t4);
		  t1 = real_a[i6];
		  real_a[i8] = add_cc(t1,t6);
		  real_a[i3] = sub_cc(t6,t1);
		  t2 = real_a[i2];
		  real_a[i4] = sub_cc(t2,t3);
		  real_a[i7] = -add_cc(t2,t3);
		  t1 = real_a[i1];
		  real_a[i1] = add_cc(t1,t5);
		  real_a[i6] = sub_cc(t1,t5);
		  t5 = real_a[i5];
		  real_a[i2] = add_cc(t5,t4);
		  real_a[i5] = sub_cc(t5,t4);
		}
	      is = sub_cc((shift_cc(FT_LEFT,id,1)),n2);
	      id = shift_cc(FT_LEFT,id,2);
	    } while (is < n1);
	}
    }
  
  
  // interlace the output and store in the output_a array
  //
  for (int_4 kk = 1; kk <= N_d/2; kk = add_cc(kk,1)){
    output_a[2*kk] = output_a[2*N_d - 2*kk] = real_a[kk];
    output_a[2*N_d - 2*kk + 1 ] = -real_a[N_d - kk];
    output_a[2*kk+1] = real_a[N_d - kk];
  }

  // manually set values to special points in the output data
  // zero imaginary part of dc, and N/2 point
  //
  output_a[0] = real_a[0];
  output_a[1] = 0.0;
  output_a[N_d+1] = 0.0;

  if ( ISIP_DEBUG_NONE < debug_level_d) {
    
    // diagnostic printing
    //
    for (int_4 kk = 0; kk < N_d; kk++) {
      printf("output[%ld] = %f    + j %f\n", kk, output_a[2*kk],
	     output_a[2*kk+1]);
    }
  }

  // set the memory used
  //
  memory_usage_d = t1.get_ref_count_cc();
  
  // clear memory
  //
  delete [] c;
  delete [] s;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}



