// file: $PDSP/class/fourier_transform/v2.0/ft_comp_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"

// method: compute_cc
//
// arguments:
// float_8* out_a : output array
// float_8* in_a : input array
//
// return: a logical_1 value indicating status
//
logical_1 Fourier_transform::compute_cc(float_8* out_a, float_8* in_a) {

  // branch on the algorithm
  //
  if (algorithm_d == FT_DFT_REAL) {
    return dft_real_cc(out_a, in_a);
  }

  if (algorithm_d == FT_DFT_COMPLEX) {
    return dft_complex_cc(out_a, in_a);
  }
  
  else if (algorithm_d == FT_RADIX_2_COMPLEX) {
    return fft2_complex_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_RADIX_2_REAL) {
    return fft2_real_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_RADIX_4_REAL) {
    return fft4_real_cc(out_a, in_a);
  }
  
  else if (algorithm_d == FT_RADIX_4_COMPLEX) {
    return fft4_complex_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_FAST_HARTLEY_COMPLEX) {
    return fht_complex_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_FAST_HARTLEY_REAL) {
    return fht_real_cc(out_a);
  }

  else if (algorithm_d == FT_SRFFT_COMPLEX) {
    return srfft_complex_cc(out_a,in_a);
   }

  else if (algorithm_d == FT_SRFFT_REAL) {
    return srfft_real_cc(out_a,in_a);
   }
  else if (algorithm_d == FT_DITF_COMPLEX) {
    return ditf_complex_cc(out_a, in_a);
  }

  else if (algorithm_d == FT_DITF_REAL) {
    return ditf_real_cc(out_a, in_a);
  }
  
  else {
    return ISIP_FALSE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
