//// file: $PDSP/class/fourier_transform/v2.0/ft_dft_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
// //#include <float_8/v1.0/float_8.h>

// method: dft_real_cc
//
// arguments:
//  float_8* out_a : output data array
//  float_8* in_a : input data array
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::dft_real_cc(float_8* out_a, float_8* in_a) {
  
  // loop over all frequency samples
  //
  for (int_4 k=0; k<N_d; k = add_cc(k,1)) {
    
    // declare local accumulators
    //
    float_8 sumr((float_8)0.0);
    float_8 sumi((float_8)0.0);
    
    // loop over the data
    //
    for(int_4 n = 0; n < N_d; n = add_cc(n,1)) {
      
      // compute the argument
      //
      float_8 arg(ISIP_TWOPI * (float_8)n * (float_8)k / (float_8)N_d);
      
      // compute the real part
      //
      sumr = mult_sum_cc(in_a[n], cos((float_8)arg), (float_8)sumr);
      
      // compute the imaginary part
      //
      sumi = mult_sum_cc(-1.0*in_a[n], sin((float_8)arg), (float_8)sumi);
    }
    
    // compute the squared magnitude
    //
    out_a[k] = (float_8)sumr*(float_8)sumr;
    out_a[k] = (float_8)sumi*(float_8)sumi + out_a[k];
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
