// file: $PDSP/class/fourier_transform/v2.0/ft_dft_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
//#include <float_8/v1.0/float_8.h>

// method: dft_complex_cc
//
// arguments:
//  float_8* out_a : output data array (used for real part of input)
//  float_8* in_a : input data array (used for imaginary part of input)
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::dft_complex_cc(float_8* out_a, float_8* in_a) {

  // define local variables
  //
  float_8 *tempout = new float_8[N_d];
  
  // loop over all frequency samples
  //
  for (int_4 k = 0; k < N_d; k = add_cc(k,1)) {
    
    // declare local accumulators
    //
    float_8 sumr((float_8)0.0);
    float_8 sumi((float_8)0.0);
    tempout[k] = 0.0;
    
    // loop over the data
    //
    for(int_4 n = 0; n < N_d; n = add_cc(n,1)) {
      
      // compute the argument
      //
      float_8 arg(ISIP_TWOPI * (float_8)n * (float_8)k / (float_8)N_d);
      
      // compute the real part
      //
      float_8 tempr = (mult_cc(in_a[n], sin(arg)));
      tempr = mult_sum_cc(out_a[n],cos(arg),tempr);
      sumr = add_cc(tempr,sumr);
      
      // compute the imaginary part
      //
      float_8 tempi = (mult_cc(in_a[n], cos(arg)));
      tempi = mult_sum_cc(out_a[n],-sin(arg),tempi);
      sumi = add_cc(tempi,sumi);
    }
    
    // compute the squared magnitude
    //
    tempout[k] = sumr*sumr;
    tempout[k] += sumi*sumi;
  }
  
  // diagnostic printing
  //
  for (int_4 kk = 0; kk < N_d; kk++){
    printf("%f \n",  tempout[kk]);
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
