//// file: $PDSP/class/fourier_transform/v2.0/ft_fft2_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
//#include <float_8/v1.0/float_8.h>

// method: fft2_real_cc
//
// This method is a fortran code converted to fit our requirements.
// The code follows the exact structure of that in the book by
// Proakis and Manolakis " Digital Signal Processing " pp.
//
// arguments:
//  float_8* output_a : output data array
//  float_8* input_a : input data array
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::fft2_real_cc( float_8* output_a,
					   float_8* input_a){
  
  // declare local variables
  //
  int_4 m,l,i1,i2;
  int_4 n1,n2;
  float_8 real_twid,imag_twid;
  float_8 tempr; 
  
  // get the lookup values
  //
  float_8 *wr = new float_8[N_d + 1];
  float_8 *wi = new float_8[N_d + 1];
  
  // create lookup table
  //
  lookup_cc((float_8*)wr,(float_8*)wi);
  
  m = powof2_cc();
  n2 = N_d;
  
  for(int_4 i = 0; i < m; i = add_cc(i,1)){
    n1 = n2;
    n2 = shift_cc(FT_RIGHT,n2,1);
    i1 = 0;
    i2 = (int_4)((N_d)/n1);
    
    for(int_4 j = 0; j < n2; j = add_cc(j,1)){
      real_twid = wr[i1];
      imag_twid = wi[i1];
      i1 = add_cc(i1,i2);
      
      for(int_4 k = j; k < N_d; k = add_cc(k,n1)){
	l = add_cc(k,n2);
	tempr = sub_cc(output_a[k],output_a[l]);
	output_a[k] = add_cc(output_a[k],output_a[l]);
        input_a[k] = add_cc(input_a[k],input_a[l]);
	output_a[l] = mult_cc(real_twid,tempr);
	input_a[l] =  mult_cc(-imag_twid,tempr);
      }
    }
  }
  
  //procedure for bit reversal
  //
  float_8 temp = 0;
  int j = 1;
  n1 = sub_cc(N_d,1);
  for (int_4 i = 1; i < n1 ; i = add_cc(i,1)){
    if (i < j)
      {
	temp = output_a[j];
	output_a[j] = output_a[i];
	output_a[i] = temp;
	temp = input_a[j];
	input_a[j] = input_a[i];
	input_a[i] = temp;
      }
    int_4 k = shift_cc(FT_RIGHT,N_d,1);
    while (k < j)
      {
	j = sub_cc(j,k);
	k = shift_cc(FT_RIGHT,k,1);
      }
    j = add_cc(j,k);
  }

 
  // delete memory
  //
  delete wr;
  delete wi;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}

