// file: $PDSP/class/fourier_transform/v2.0/ft_fft2_1.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
//#include <float_8/v1.0/float_8.h>

// method: fft2_complex_cc
//
// This method is a fortran code converted to fit our requirements.
// The code follows the exact structure of that in the book by
// Proakis and Manolakis " Digital Signal Processing " pp.
//
// arguments:
//  float_8* in_r_a : output real array
//  float_8* in_i_a : input imaginary array
//
// return: a logical_1 value indicating status
//
// implements a real discrete fourier transform
//
logical_1 Fourier_transform::fft2_complex_cc( float_8* in_r_a,
					      float_8* in_i_a){
  

  // declare local variables
  //
  int_4 m,l,i1,i2;
  int_4 n1,n2;
  float_8 real_twid,imag_twid;
  float_8 tempr, tempi; 
  
  // get the lookup values
  //
  float_8 *wr = new float_8[N_d];
  float_8 *wi = new float_8[N_d];
   
  // create lookup table
  //
  lookup_cc((float_8*)wr,(float_8*)wi);
  
  m = powof2_cc();
  n2 = N_d;
  
  for(int_4 i = 0; i < m; i = add_cc(i,1)){
    n1 = n2;
    n2 = shift_cc(FT_RIGHT,n2,1);
    i1 = 0;
    i2 = (int_4)((N_d)/(float_8)n1);
    
    for(int_4 j = 0; j < n2; j = add_cc(j,1)){
      real_twid = wr[i1];
      imag_twid = wi[i1];
      i1= add_cc(i1,i2);
      
      for(int_4 k = j; k < N_d; k = add_cc(k,n1)){
	l = add_cc(k,n2);
	tempr = sub_cc(in_r_a[k],in_r_a[l]);
	in_r_a[k] = add_cc(in_r_a[k],in_r_a[l]);
	tempi = sub_cc(in_i_a[k],in_i_a[l]);
	in_i_a[k] = add_cc(in_i_a[k],in_i_a[l]);
	in_r_a[l] = add_cc(mult_cc(real_twid,tempr),
			   mult_cc(imag_twid,tempi));
	in_i_a[l] = sub_cc(mult_cc(real_twid,tempi),
			   mult_cc(imag_twid,tempr));
      }
    }
  }

  //procedure for bit reversal
  //
  float_8 temp = 0;
  int_4 j = 1;
  n1 = sub_cc(N_d,1);
  for (int_4 i = 1; i < n1; i = add_cc(i,1)){
    if (i < j)
      {
	temp = in_r_a[j];
	in_r_a[j] = in_r_a[i];
	in_r_a[i] = temp;
	temp = in_i_a[j];
	in_i_a[j] = in_i_a[i];
	in_i_a[i] = temp;
      }
    int_4 k = shift_cc(FT_RIGHT,N_d,1);
    while (k < j){
      j = sub_cc(j,k);
      k = shift_cc(FT_RIGHT,k,1);
    }
    j = add_cc(j,k);
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}

