// file: $PDSP/class/fourier_transform/v2.0/ft_fht_0.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
//#include <float_8/v1.0/float_8.h>
#include "ft_fht_0.h"

#define GOOD_TRIG

// method: fht_real_cc
//
// This code is based on the algorithm as described in the work by
// Ron Mayer whose code is available as a free-ware. The resulting array
// will have first half containing the real part  and the second half the
// imaginary part. This can be done because the fourier transform is
// symmetric.
//
// arguments:
//  float_8* real_a : data array
//
// return: a logical_1 value indicating status
//
// implements a real Fast Hartley transform
//
logical_1 Fourier_transform::fht_real_cc(float_8* real_a){
  
  // declare local variables
  float_8 a,b;
  float_8 c1,s1,s2,c2,s3,c3,s4,c4;
  float_8 f0,g0,f1,g1,f2,g2,f3,g3;
  int_4 i,k,k1,k2,k3,k4,kx;
  float_8 *fi,*fn,*gi;
  TRIG_VARS;

  for (k1 = 1,k2 = 0; k1 < N_d; k1 = add_cc(k1,1)){
    for (k=shift_cc(FT_RIGHT,N_d,1);(!((k2^=k)&k));
	 k = shift_cc(FT_RIGHT,k,1));
    if (k1>k2){
      a=real_a[k1];real_a[k1]=real_a[k2];real_a[k2]=a;
    }
  }
  for ( k=0 ; (shift_cc(FT_LEFT,1,k))<N_d ; k = add_cc(k,1)){};
  k  &= 1;
  if (k==0){
    for (fi = real_a,fn = real_a + N_d; fi <fn; fi += 4){
      f1 = sub_cc(fi[0 ],fi[1 ]);
      f0 = add_cc(fi[0 ],fi[1 ]);
      f3 = sub_cc(fi[2 ],fi[3 ]);
      f2 = add_cc(fi[2 ],fi[3 ]);
      fi[2] = sub_cc(f0,f2);	
      fi[0] = add_cc(f0,f2);
      fi[3] = sub_cc(f1,f3);	
      fi[1] = add_cc(f1,f3);
    }
  }

  else {
    
    for (fi = real_a,fn = real_a + N_d, gi = fi + 1;
	 fi < fn; fi += 8,gi += 8){
      c1 = sub_cc(fi[0 ],gi[0 ]);
      s1 = add_cc(fi[0 ],gi[0 ]);
      c2 = sub_cc(fi[2 ],gi[2 ]);
      s2 = add_cc(fi[2 ],gi[2 ]);
      c3 = sub_cc(fi[4 ],gi[4 ]);
      s3 = add_cc(fi[4 ],gi[4 ]);
      c4 = sub_cc(fi[6 ],gi[6 ]);
      s4 = add_cc(fi[6 ],gi[6 ]);
      f1 = sub_cc(s1,s2);	
      f0 = add_cc(s1,s2);
      g1 = sub_cc(c1,c2);	
      g0 = add_cc(c1,c2);
      f3 = sub_cc(s3,s4);	
      f2 = add_cc(s3,s4);
      g3 = mult_cc(SQRT2,c4);		
      g2 = mult_cc(SQRT2,c3);
      fi[4 ] = sub_cc(f0,f2);
      fi[0 ] = add_cc(f0,f2);
      fi[6 ] = sub_cc(f1,f3);
      fi[2 ] = add_cc(f1,f3);
      gi[4 ] = sub_cc(g0,g2);
      gi[0 ] = add_cc(g0,g2);
      gi[6 ] = sub_cc(g1,g3);
      gi[2 ] = add_cc(g1,g3);
    }
  }

  if (N_d<16) 

    // exit gracefully
    //
    return ISIP_TRUE;

  do {
     k  = add_cc(2,k);
     k1 = shift_cc(FT_LEFT,1,k);
     k2 = shift_cc(FT_LEFT,k1,1);
     k4 = shift_cc(FT_LEFT,k2,1); 
     k3 = add_cc(k2,k1);
     kx = shift_cc(FT_RIGHT,k1,1); 
     fi = real_a;
     gi = add_cc(fi,kx);
     fn = add_cc(real_a,N_d);
     do {
       f1 = sub_cc(fi[0 ],fi[k1]);
       f0 = add_cc(fi[0 ],fi[k1]);
       f3 = sub_cc(fi[k2],fi[k3]);
       f2 = add_cc(fi[k2],fi[k3]);
       fi[k2] = sub_cc(f0,f2);
       fi[0 ] = add_cc(f0,f2);
       fi[k3] = add_cc(f1,f3);
       fi[k1] = add_cc(f1,f3);
       g1 = sub_cc(gi[0 ],gi[k1]);
       g0 = add_cc(gi[0 ],gi[k1]);
       g3 = mult_cc(SQRT2,gi[k3]);
       g2 = mult_cc(SQRT2,gi[k2]);
       gi[k2] = sub_cc(g0,g2);
       gi[0 ] = add_cc(g0,g2);
       gi[k3] = sub_cc(g1,g3);
       gi[k1] = add_cc(g1,g3);
       gi = add_cc(gi,k4);
       fi = add_cc(fi,k4);
     } while (fi < fn);
     
     TRIG_INIT(k,c1,s1);
     for (i = 1; i < kx; i = add_cc(i,1)){
       TRIG_NEXT(k,c1,s1);
       c2 = sub_cc(mult_cc(c1,c1),mult_cc(s1,s1));
       s2 = mult_cc(2,mult_cc(c1,s1));
       fn = add_cc(real_a,N_d);
       fi = add_cc(real_a,i);
       gi = add_cc(real_a, sub_cc( k1,i));
       do{
	 b  = sub_cc(mult_cc(s2,fi[k1]),mult_cc(c2,gi[k1]));
	 a  = add_cc(mult_cc(c2,fi[k1]),mult_cc(s2,gi[k1]));
	 f1 = sub_cc(fi[0 ],a);
	 f0 = add_cc(fi[0 ],a);
	 g1 = sub_cc(gi[0 ],b);
	 g0 = add_cc(gi[0 ],b);
	 b  = sub_cc(mult_cc(s2,fi[k3]),mult_cc(c2,gi[k3]));
	 a  = add_cc(mult_cc(c2,fi[k3]),mult_cc(s2,gi[k3]));
	 f3 = sub_cc(fi[k2],a);
	 f2 = add_cc(fi[k2],a);
	 g3 = sub_cc(gi[k2],b);
	 g2 = add_cc(gi[k2],b);
	 b  = sub_cc(mult_cc(s1,f2),mult_cc(c1,g3));
	 a  = add_cc(mult_cc(c1,f2),mult_cc(s1,g3));
	 fi[k2]  = sub_cc(f0,a);
	 fi[0 ]  = add_cc(f0,a);
	 gi[k3]  = sub_cc(g1,b);
	 gi[k1]  = add_cc(g1,b);
	 b  = sub_cc(mult_cc(c1,g2),mult_cc(s1,f3));
	 a  = add_cc(mult_cc(s1,g2),mult_cc(c1,f3));
	 gi[k2]  = sub_cc(g0,a);
	 gi[0 ]  = add_cc(g0,a);
	 fi[k3]  = sub_cc(f1,b);
	 fi[k1]  = add_cc(f1,b);
	 gi = add_cc(gi,k4);
	 fi = add_cc(fi,k4);
       } while (fi < fn);
     }
     TRIG_RESET(k,c1,s1);
   } while (k4<N_d);
  
  // exit gracefully
  //
  return ISIP_TRUE;
}





