// file: $PDSP/class/fourier_transform/v2.0/ft_fht_1.cc
//

// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
//#include <float_8/v1.0/float_8.h>

// method: fht_complex_cc
//
// This code is based on the algorithm as described in the work by
// Ron Mayer whose code is available as a free-ware. 
//
// arguments:
//  float_8* real_a : real data array
//  float_8* imag_a : imag data array
//
// return: a logical_1 value indicating status
//
// implements a complex Fast Hartley transform
//
logical_1 Fourier_transform::fht_complex_cc(float_8* real_a, float_8* imag_a) {

  // declare local variables
  //
  float_8 a,b,c,d;
  int_4 i,j,k;
  float_8 q,r,s,t;

  // prepare data for FHT computations
  //
  for (i = 1,j = sub_cc(N_d,1), k = (N_d/2); i < k; i = add_cc(i,1),j = sub_cc(j,1)) {
    a = real_a[i];
    b = real_a[j];
    q = add_cc(a,b);
    r = sub_cc(a,b);
    c = imag_a[i];
    d = imag_a[j];
    s = add_cc(c,d);
    t = sub_cc(c,d);
    real_a[i] = mult_cc(add_cc(q,t),0.5);
    real_a[j] = mult_cc(sub_cc(q,t),0.5);
    imag_a[i] = mult_cc(sub_cc(s,r),0.5);
    imag_a[j] = mult_cc(add_cc(s,r),0.5);
  }

  // compute hartley transform of the real and imaginary parts separately
  //
  fht_real_cc(real_a);
  fht_real_cc(imag_a);
  
  // exit gracefully
  //
  return ISIP_TRUE;
}



