// file: $PDSP/class/fourier_transform/v2.0/ft_srfft_0.cc
//
 
// isip include files
//
#include "fourier_transform.h"
#include "fourier_transform_constants.h"
 //#include <float_8/v1.0/float_8.h>

#define DSQC    0.707106781186547f

// method: srfft_real_cc
//
// This code is based on the code by Sitton of the Rice University.
//
// arguments:
//  float_8* real_a : real part of input/output data
//  float_8* imag_a : imaginary part of output data
//
// return : logical_1 indicating no error in computation
//
// routine used to compute the real  split-radix FFT
//
logical_1 Fourier_transform::srfft_real_cc(float_8 *real_a, float_8 *imag_a) {

  float_8   t1, t2, t3, t4, t5, t6;
  float_8   cc1, ss1, cc3, ss3;
  int_4     i, j, k, l, n1, n2, n4, n8, inc;
  int_4     is, id, i1, i2, i3, i4, i5, i6, i7, i8;

  // create sin-cos lookup table
  //
  float_8* c = new float_8[N_d];
  float_8* s = new float_8[N_d];

  srfft_new_lookup_cc((float_8*)c,(float_8*)s);

  int_4 m = powof2_cc();
  
  //  First (twiddleless) L Butterflies   
  //
  is = 0;
  id = 4;
  n1 = sub_cc(N_d,1);

  do {
      for (i = is; i < n1; i = add_cc(i,id)){
	  t1 = real_a[i];
	  i1 = add_cc(i,1);
	  t2 = real_a[i1];
	  real_a[i] = add_cc(t1,t2);
	  real_a[i1] = sub_cc(t1,t2);
	}
      is = sub_cc((shift_cc(FT_LEFT,id,1)),2);
      id = shift_cc(FT_LEFT,id,2);
    } while (is < n1) ;
  
  //  Other (twiddled) L Butterflies  
  //
  n2 = 2;
  for (k = 2; k <= m; k = add_cc(k,1)){
      n4 = shift_cc(FT_RIGHT,n2,1);
      n8 = shift_cc(FT_RIGHT,n4,1);
      n2 = shift_cc(FT_LEFT,n2,1);
      is = 0;
      id = shift_cc(FT_LEFT,n2,1);

      do {
	  for (i = is; i < N_d; i = add_cc(i,id)){
	      i2 = (add_cc(i,n4));
	      i3 = (add_cc(i2,n4));
	      i4 = (add_cc(i3,n4));
	      t3 = real_a[i3];
	      t4 = real_a[i4];
	      t1 = real_a[i];
	      t2 = add_cc(t4,t3);
	      real_a[i4] = sub_cc(t4,t3);
	      real_a[i3] = sub_cc(t1,t2);
	      real_a[i] = add_cc(t1,t2);
	      
	      if (n4 != 1) {
		  i1 = add_cc(i,n8);
		  i2 = add_cc(n8,i2);
		  i3 = add_cc(n8,i3);
		  i4 = add_cc(n8,i4);
		  t3 = real_a[i3];
		  t4 = real_a[i4];
		  t1 = mult_cc(add_cc(t3,t4),DSQC);
		  t2 = mult_cc(sub_cc(t3,t4),DSQC);
		  t3 = real_a[i1];
		  t4 = real_a[i2];
		  real_a[i4] = sub_cc(t4,t1);
		  real_a[i3] = -add_cc(t4,t1);
		  real_a[i2] = sub_cc(t3,t2);
		  real_a[i1] = add_cc(t3,t2);
		}
	    }
	  is = sub_cc(shift_cc(FT_LEFT,id,1),n2);
	  id = shift_cc(FT_LEFT,id,2);
	} while (is < n1);
      
      inc = N_d/n2;
      l = inc;
      for (j = 1; j < n8; j = add_cc(j,1)){
	  cc1 = c[l];
	  ss1 = s[l];
	  i1 = mult_cc(l,3);
	  cc3 = c[i1];
	  ss3 = s[i1];
	  l = mult_cc(add_cc(j,1),inc);
	  is = 0;
	  id = shift_cc(FT_LEFT,n2,1);
	  do
	    {
	      for (i = is; i < N_d; i = add_cc(i,id)){
		  i1 = add_cc(i, j);
		  i2 = add_cc(i1, n4);
		  i3 = add_cc(i2, n4);
		  i4 = add_cc(i3, n4);
		  i5 = add_cc(sub_cc(i,j),n4);
		  i6 = add_cc(i5, n4);
		  i7 = add_cc(i6, n4);
		  i8 = add_cc(i7, n4);
		  t3 = real_a[i3];
		  t4 = real_a[i7];
		  t1 = add_cc(mult_cc(t3,cc1),mult_cc(t4,ss1));
		  t2 = sub_cc(mult_cc(t4,cc1),mult_cc(t3,ss1));
		  t5 = real_a[i4];
		  t6 = real_a[i8];
		  t3 = add_cc(mult_cc(t5,cc3),mult_cc(t6,ss3));
		  t4 = sub_cc(mult_cc(t6,cc3),mult_cc(t5,ss3));
		  t5 = add_cc(t1, t3);
		  t3 = sub_cc(t1,t3);
		  t6 = add_cc(t2,t4);
		  t4 = sub_cc(t2,t4);
		  t1 = real_a[i6];
		  real_a[i8] = add_cc(t1,t6);
		  real_a[i3] = sub_cc(t6,t1);
		  t2 = real_a[i2];
		  real_a[i4] = sub_cc(t2,t3);
		  real_a[i7] = -add_cc(t2,t3);
		  t1 = real_a[i1];
		  real_a[i1] = add_cc(t1,t5);
		  real_a[i6] = sub_cc(t1,t5);
		  t5 = real_a[i5];
		  real_a[i2] = add_cc(t5,t4);
		  real_a[i5] = sub_cc(t5,t4);
		}
	      is = sub_cc((shift_cc(FT_LEFT,id,1)),n2);
	      id = shift_cc(FT_LEFT,id,2);
	    } while (is < n1);
	}
    }

   
  // clear memory
  //
  delete c;
  delete s;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}



