// file: $PDSP/util/compare_fft/v1.1/cf_get_0.cc
//

// isip include files
//
#include <fourier_transform.h>
#include <fourier_transform_constants.h>
#include "compare_fft.h"

// method: get_params_cc
//
// arguments:
//  int argc: argc from the main program
//  char** argv: argv from the main program
//  int_4& order_a: fft order ( <= 8192 )
//  char_1* data_type_a: type of input data ( real/complex )
//  int_4& num_iter_a: num_of_iterations
//  int_4& dbg_level_a: the debug level
//  int_4& stats_a: number of statistics ( time , number of computations
//                                              etc.)
//  char_1* algo_name_a: name of the algorithm
//
// return: a logical_1 value indicating status
//
logical_1 get_params_cc(int argc, char** argv,
			int_4& order_a,char_1* data_type_a,
			int_4& num_iter_a,
			int_4& dbg_level_a,
			int_4& stats_a, char_1* algo_name_a
			) {

  // declare local variable
  //
  char_1* command_string = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // decoding loop
  //
  int_4 last_index = argc;
  
  for(int_4 index=1; index<last_index; index++) {

    // pick off order of FFT
    //
    if (strcmp((char*)OPT_ORDER,argv[index]) == 0) {
      order_a = atoi(argv[index+1]);
    }

    // pick off input data type
    //
    else if (strcmp((char*)OPT_DATA_TYPE,argv[index]) == 0) {
      strcpy((char*)data_type_a,argv[index+1]);
    }
    
    // pick off number of iterations
    //
    else if (strcmp((char*)OPT_NUM_ITER,argv[index]) == 0) {
      num_iter_a = atoi(argv[index+1]);
    }

    // pick off debug level
    //
    else if (strcmp((char*)OPT_DBG_LEVEL,argv[index]) == 0) {
      dbg_level_a = atoi(argv[index+1]);
    }

    // pick off amount of statistics required
    //
    else if (strcmp((char*)OPT_STATS,argv[index]) == 0) {
      stats_a = atoi(argv[index+1]);
    }

    // pick off alogrithm to be used
    //
    else if (strcmp((char*)OPT_ALGO_NAME,argv[index]) == 0) {
      strcpy((char*)algo_name_a,argv[index+1]);
    }

    // if help asked for display the help screen
    //
    else if (strcmp((char*)OPT_HELP,argv[index]) == 0) {
      sprintf((char*)command_string,(char*)ISIP_HELP_FORMAT,
	      (char*)COMPARE_FFT_HELP_FILE_NAME);
      system((char*)command_string);
      
      // exit gracefully
      //
      return ISIP_FALSE;
    }    
  }

  // free memory
  //
  delete [] command_string;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
  
