// file: $PDSP/util/fourier_transform/v1.1/compare_fft.cc
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// isip include files
//
#include <fourier_transform.h>
#include <fourier_transform_constants.h>
#include "compare_fft.h"


// main driver for the application - command line driver to test the various
// fft algorithms
//
main (int argc, char** argv) {

  // declare local variables
  //
  char_1* data_type = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* algo_string = new char_1[ISIP_MAX_STRING_LENGTH];
  int_4 order, stats, dbg_level, num_iter, algo_name;

  // assign default values
  //
  num_iter = (int_4)DEFAULT_NUM_ITER;
  stats = (int_4)DEFAULT_STATS;
  dbg_level = (int_4)DEFAULT_DBG_LEVEL;
  order = (int_4)DEFAULT_ORDER;
  strcpy((char*)data_type,(char*)DEFAULT_DATA_TYPE);
  strcpy((char*)algo_string,(char*)DEFAULT_ALGO_STRING);
  algo_name = DEFAULT_ALGO_NAME;
  
 
  // declare a fourier transform object
  //
  Fourier_transform ft;

  // get the parameters from the command line
  //
  if (get_params_cc(argc,argv,order,
		    data_type, num_iter,
		    dbg_level, stats,
		    algo_string) == ISIP_FALSE ){
    return ISIP_NO_ERROR;
  }

  // declare memory for input and output data
  // since the data is assumed to be interlaced, the array size
  // should be twice the order for a complex signal
  //
  float_4* input = new float_4[2*order];
  float_4* output = new float_4[2*order];

  // set the debug_level
  //
  ft.set_debug_cc(dbg_level);
  
  // map the algorithm name to its corresponding number as a fourier
  // transform constant
  //
  algo_name = get_algo_number_cc(algo_string);
  
  for (int_4 i = 0; i < 4; i++){
    input[2*i] = i;
    input[2*i + 1] = 0;
  }
  
  for (int_4 i = 4; i < order; i++){
    input[2*i] = i;
    input[2*i + 1] = 0;
  }
  
  // set the parameters for the fourier transform object
  //
  if (strcmp((char*)data_type,"real") == 0){
    if (ft.set_cc(order,algo_name,FT_REAL) == ISIP_FALSE)
      return ISIP_ERROR;
  }
  else if(strcmp((char*)data_type,"complex") == 0){
    if (ft.set_cc(order,algo_name,FT_COMPLEX) == ISIP_FALSE)
      return ISIP_ERROR;
  }
  
  // reset the object parameters
  //
  ft.reset_stats_cc();

  // compute the fourier transform of the input using the chosen fft
  // algorithm
  //

  // begin timing the computations
  //
  ft.set_cputime_cc(clock());

  // compute the fft for the number of iterations provided
  //
  for (int_4 i = 0; i < num_iter ; i++){
    ft.compute_cc(output, input);
  }
  
  // get the timing information
  //
  ft.set_cputime_cc(clock());
  
  // print the statistics of the algorithm
  //
  ft.display_stats_cc(stdout,num_iter,stats);
  
  // free memory
  //
  delete [] data_type;
  delete [] input;
  delete [] output;
  delete [] algo_string;
  
  // exit gracefully
  //
  return ISIP_NO_ERROR;
}







